////////////////////////////////////////////////////////////////////////////////////////
//
// Nestopia - NES/Famicom emulator written in C++
//
// Copyright (C) 2003-2006 Martin Freij
//
// This file is part of Nestopia.
// 
// Nestopia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// Nestopia is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Nestopia; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
////////////////////////////////////////////////////////////////////////////////////////

#ifndef NST_INPUT_DEVICE_H
#define NST_INPUT_DEVICE_H

#ifdef NST_PRAGMA_ONCE_SUPPORT
#pragma once
#endif

#include "../NstState.hpp"
#include "../api/NstApiInput.hpp"

namespace Nes
{
	namespace Core
	{
		namespace Input
		{
			class Device
			{
			protected:

				static bool Strobe(uint& latch,uint toggle)
				{
					toggle &= 0x01;
					const bool record = (latch < toggle);
					latch = toggle;
					return record;
				}

				typedef Api::Input::Type Type;

				const Type type;
				Controllers* input;
		
			public:
		
				Device(Type t=Api::Input::UNCONNECTED)
				: type(t), input(NULL) {}
		
				virtual ~Device() {}

				virtual void Initialize(dword) {}
				virtual void Reset() {}
				virtual void LoadState(State::Loader&,dword) {}
				virtual void SaveState(State::Saver&,uchar) const {}
		
				virtual void BeginFrame(Controllers* i)
				{ 
					input = i;
				}
		
				virtual void Poke(uint) 
				{
				}
		
				virtual uint Peek(uint) 
				{
					return 0;
				}
		
				Type GetType() const
				{ 
					return type; 
				}
			};
		}
	}
}

#endif
