////////////////////////////////////////////////////////////////////////////////////////
//
// Nestopia - NES/Famicom emulator written in C++
//
// Copyright (C) 2003-2006 Martin Freij
//
// This file is part of Nestopia.
// 
// Nestopia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// Nestopia is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Nestopia; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
////////////////////////////////////////////////////////////////////////////////////////

#include "NstWindowMenu.hpp"
#include "NstDialogBarcodeReader.hpp"
#include "NstManagerBarcodeReader.hpp"

namespace Nestopia
{
	using namespace Managers;

	BarcodeReader::BarcodeReader(Emulator& e,Window::Menu& m)
	: emulator(e), menu(m)
	{
		m.Commands().Add( IDM_MACHINE_EXT_BARCODE, this, &BarcodeReader::OnMenuCmd );
		emulator.Events().Add( this, &BarcodeReader::OnEmuEvent );
	}

	BarcodeReader::~BarcodeReader()
	{
		emulator.Events().Remove( this );
	}

	void BarcodeReader::OnMenuCmd(uint)
	{
		Window::BarcodeReader dialog( emulator, lastCode );
	}

	void BarcodeReader::OnEmuEvent(Emulator::Event event)
	{
		switch (event)
		{
			case Emulator::EVENT_POWER_ON:
			case Emulator::EVENT_POWER_OFF:
		
				menu[IDM_MACHINE_EXT_BARCODE].Enable
				(
			       	event == Emulator::EVENT_POWER_ON && 
					Nes::BarcodeReader(emulator).IsConnected() 
				);
				break;
		}
	}
}
